local M = {}

local running = true
local scenario = nil
local speed = 0
local crash = 0

local helper = require('scenario/scenariohelper')

local function reset()
    running = false
	crash = 0
end

local function fail(reason)
    scenarios.finish({failed = reason})
    reset()
end

local function success(speed)
    local result = {msg = 'Total damage done: $'..string.format("%0.2f", speed)..''}
    scenarios.finish(result)
    reset()
end

local function onRaceStart()
    reset()

    helper.init()
    helper.setAiMode('aiCar1', 'manual')
    helper.setAiAggression('aiCar1', 0.2)
    helper.setAiTarget('aiCar1', 'CoolidgeL_AQ')
    helper.trackVehicle('aiCar1', 'aiCarA')
	
	helper.setAiMode('aiCar2', 'manual')
    helper.setAiAggression('aiCar2', 0.5)
    helper.setAiTarget('aiCar2', 'CoolidgeR_AQ')
    helper.trackVehicle('aiCar2', 'aiCarB')
	
	helper.setAiMode('aiCar3', 'manual')
    helper.setAiAggression('aiCar3', 3.8)
    helper.setAiTarget('aiCar3', 'CoolidgeR_AP')
    helper.trackVehicle('aiCar3', 'aiCarC')

    helper.trackVehicle('scenario_player0', 'playerCar')

    running = true
end

local function onRaceTick(raceTickTime)
    if not running then
        return
    end

    helper.update(raceTickTime)

    local aiCarAData = map.objects[map.objectNames['aiCarA']]
	local aiCarBData = map.objects[map.objectNames['aiCarB']]
	local aiCarCData = map.objects[map.objectNames['aiCarC']]
    local playerCarData = map.objects[map.objectNames['playerCar']]

    if not aiCarAData or not playerCarData or not aiCarBData or not aiCarCData then
        return
    end
	
	if crash >= 1 then
		helper.queueLuaCommandByName('scenario_player0', 'bullettime.set(2)')
    end
	
	if playerCarData.damage >= 10000 then
		fail("You crashed your car!")
	end

    if aiCarAData.damage >= 500.0 then
		helper.setAiMode('aiCar1', 'disabled')
		crash = crash + 1
    end
	
	if aiCarBData.damage >= 50.0 then
		helper.setAiMode('aiCar2', 'disabled')
		crash = crash + 1
    end
	
	if aiCarCData.damage >= 500.0 then
		helper.setAiMode('aiCar3', 'disabled')
		crash = crash + 1
    end

    if scenario.timer > 25 then
		local playerCar = helper.getVehicleByName('scenario_player0')
		if crash == 0 then
			fail("You lost!")
		elseif crash >= 1 then
			trafficDamage = ((aiCarAData.damage + aiCarBData.damage + aiCarCData.damage) * 0.1) * crash
			playerDamage = playerCarData.damage * 0.5
			totalDamage = trafficDamage - playerDamage
			if totalDamage < 500000 then
				fail("You didn't do enough damage!")
			elseif totalDamage >= 500000 then
				success(totalDamage)
			end
		end
    end
end

local function onScenarioChange(sc)
    scenario = sc
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onScenarioChange = onScenarioChange

return M