local M = {}

local running = true
local scenario = nil
local speed = 0
local resetA = 0
local resetB = 0
local resetC = 0

local helper = require('scenario/scenariohelper')

local function reset()
    running = false
	resetA = 0
	resetB = 0
	resetC = 0
end

local function resetPos()
	TorqueScript.eval('aiCar1.setTransform("-23.5776 -67.7671 17.9693");')
	TorqueScript.eval('aiCar2.setTransform("12.5567 -72 16.5793");')
	TorqueScript.eval('aiCar3.setTransform("-79.702 -72 23.9162");')
end

local function fail(reason)
    scenarios.finish({failed = reason})
    reset()
end

local function success(speed)
    local result = {msg = 'Total damage done: $'..string.format("%0.2f", speed)..''}
    scenarios.finish(result)
    reset()
	resetPos()
end

local function onRaceStart()
    reset()
	resetPos()

    helper.init()
    helper.setAiMode('aiCar1', 'manual')
    helper.setAiAggression('aiCar1', 3.5)
    helper.setAiTarget('aiCar1', 'CoolidgeL_AQ')
    helper.trackVehicle('aiCar1', 'aiCarA')
	
	helper.setAiMode('aiCar2', 'manual')
    helper.setAiAggression('aiCar2', 3.5)
    helper.setAiTarget('aiCar2', 'CoolidgeR_AQ')
    helper.trackVehicle('aiCar2', 'aiCarB')
	
	helper.setAiMode('aiCar3', 'manual')
    helper.setAiAggression('aiCar3', 3.5)
    helper.setAiTarget('aiCar3', 'CoolidgeR_AQ')
    helper.trackVehicle('aiCar3', 'aiCarC')

    helper.trackVehicle('scenario_player0', 'playerCar')

    running = true
end

local function onRaceTick(raceTickTime)
    if not running then
        return
    end

    helper.update(raceTickTime)

    local aiCarAData = map.objects[map.objectNames['aiCarA']]
	local aiCarBData = map.objects[map.objectNames['aiCarB']]
	local aiCarCData = map.objects[map.objectNames['aiCarC']]
    local playerCarData = map.objects[map.objectNames['playerCar']]

    if not aiCarAData or not playerCarData or not aiCarBData or not aiCarCData then
        return
    end
	
	local playerDistance = helper.getDistanceBetweenSceneObjects('scenario_player0', 'wp01')
	local carADistance = helper.getDistanceBetweenSceneObjects('aiCar1', 'wp01')
	local carBDistance = helper.getDistanceBetweenSceneObjects('aiCar2', 'wp01')
	local carCDistance = helper.getDistanceBetweenSceneObjects('aiCar3', 'wp01')
	
	if playerCarData.damage >= 30000 then
		fail("You crashed your car!")
	end

    if aiCarAData.damage >= 5000.0 then
		helper.setAiMode('aiCar1', 'stop')
    end
	
	if carADistance >= (playerDistance + 30) and resetA == 0 then
		TorqueScript.eval('aiCar1.setTransform("-179.944 -67.7671 30.4011");')
		helper.setAiMode('aiCar1', 'manual')
		helper.setAiTarget('aiCar1', 'CoolidgeR_AQ')
		resetA = 1
	end
	
	if aiCarBData.damage >= 5000.0 then
		helper.setAiMode('aiCar2', 'stop')
    end
	
	if carBDistance >= (playerDistance + 100) and resetB == 0 then
		TorqueScript.eval('aiCar2.setTransform("-167.944 -72.7671 30.4011");')
		helper.setAiMode('aiCar2', 'manual')
		helper.setAiTarget('aiCar2', 'CoolidgeR_AQ')
		resetB = 1
	end
	
	if aiCarCData.damage >= 10000.0 then
		helper.setAiMode('aiCar3', 'stop')
    end
	
	if carCDistance >= (playerDistance + 50) and resetC == 0 then
		TorqueScript.eval('aiCar3.setTransform("-179.944 -67.7671 30.8011");')
		helper.setAiMode('aiCar3', 'manual')
		helper.setAiTarget('aiCar3', 'CoolidgeL_AQ')
		resetC = 1
	end

    if scenario.timer > 20 then
		fail("You lost!")
    end
end

local function onScenarioChange(sc)
    scenario = sc
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onScenarioChange = onScenarioChange

return M