local M = {}

local running = true
local scenario = nil
local speed = 0
local crash = 0

local vehicle = nil

local helper = require('scenario/scenariohelper')


local function reset()
    running = false
	crash = 0
	helper.queueLuaCommandByName('scenario_player0', 'bullettime.set(2)')
	--helper.queueLuaCommandByName('scenario_player0', 'bullettime.set(2)')
end

local function fail(reason)
    scenarios.finish({failed = reason})
    reset()
end

local function success(speed)
    local result = {msg = 'Total damage done: $'..string.format("%0.2f", speed)..''}
    scenarios.finish(result)
    reset()
end

local function onRaceStart()
    reset()

    helper.init()
    helper.setAiMode('aiCar1', 'manual')
    helper.setAiAggression('aiCar1', 0.8)
    helper.setAiTarget('aiCar1', 'LombardEB_AA')
    helper.trackVehicle('aiCar1', 'aiCarA')
	
	helper.setAiMode('aiCar2', 'manual')
    helper.setAiAggression('aiCar2', 0.5)
    helper.setAiTarget('aiCar2', 'LombardWB_AD')
    helper.trackVehicle('aiCar2', 'aiCarB')
	
	helper.setAiMode('aiCar3', 'manual')
    helper.setAiAggression('aiCar3', 1.5)
    helper.setAiTarget('aiCar3', 'LombardEB_AA')
    helper.trackVehicle('aiCar3', 'aiCarC')
	
	helper.trackVehicle('aiCar4', 'aiCarD')

    helper.trackVehicle('scenario_player0', 'playerCar')
	
	--helper.queueLuaCommandByName('scenario_player0', 'bullettime.set(2)')

    running = true
end

local function onRaceTick(raceTickTime)
    if not running then
        return
    end

    helper.update(raceTickTime)

    local aiCarAData = map.objects[map.objectNames['aiCarA']]
	local aiCarBData = map.objects[map.objectNames['aiCarB']]
	local aiCarCData = map.objects[map.objectNames['aiCarC']]
	local aiCarDData = map.objects[map.objectNames['aiCarD']]
    local playerCarData = map.objects[map.objectNames['playerCar']]

    if not aiCarAData or not playerCarData or not aiCarBData or not aiCarCData then
        return
    end

    if aiCarAData.damage > 0 and speed == 0 then
        speed = playerCarData.vel:length()
    end
	
	if playerCarData.damage >= 50.0 and crash == 1 then
		helper.queueLuaCommandByName('scenario_player0', 'bullettime.set(4)')
    end
	
    if aiCarAData.damage >= 50.0 then
		helper.setAiMode('aiCar1', 'stop')
		crash = crash + 1
    end
	
	if aiCarBData.damage >= 50.0 then
		helper.setAiMode('aiCar2', 'stop')
		crash = crash + 1
    end
	
	if aiCarCData.damage >= 50.0 then
		helper.setAiMode('aiCar3', 'stop')
		crash = crash + 1
    end
	
	if aiCarDData.damage >= 50.0 then
		crash = crash + 1
    end

    if scenario.timer > 30 then
		if crash == 0 then
			fail("You lost!")
		elseif crash >= 2 then
			totalDamage = ((aiCarAData.damage + aiCarBData.damage + aiCarCData.damage + aiCarDData.damage + playerCarData.damage) * 0.1) * crash
			if totalDamage < 1500000 then
				fail("You didn't do enough damage!")
			elseif totalDamage >= 1500000 then
				success(totalDamage)
			end
		end
    end
end

local function onScenarioChange(sc)
    scenario = sc
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onScenarioChange = onScenarioChange

return M